import 'package:flutter/cupertino.dart';
import 'package:forme/forme.dart';

import 'package:forme_cupertino_fields/forme_cupertino_fields.dart';

typedef Updater<T> = T Function(T t);

abstract class BasePage<T, E extends FormeModel> extends StatelessWidget {
  final String name = 'field';
  final FormeKey formKey = FormeKey();

  FormeValueFieldController<T, E> get controller => formKey.valueField(name);

  String get title;

  Widget get body;

  Widget createButton(String text, VoidCallback onPressed) {
    return Padding(
      padding: EdgeInsets.all(5),
      child: CupertinoButton(onPressed: onPressed, child: Text(text)),
    );
  }

  Widget builderButton(
      String text, void Function(BuildContext context) onPressed) {
    return Builder(builder: (context) {
      return Padding(
        padding: EdgeInsets.all(5),
        child: CupertinoButton(
            onPressed: () {
              onPressed(context);
            },
            child: Text(text)),
      );
    });
  }

  void updateLabel() {
    controller.decoratorController.update(
        FormeCupertinoInputDecoratorModel(prefix: Text('New Label Text')));
  }

  void updateHelper() {
    controller.decoratorController
        .update(FormeCupertinoInputDecoratorModel(helper: Text('helper')));
  }

  @override
  Widget build(BuildContext context) {
    return CupertinoPageScaffold(
      resizeToAvoidBottomInset: false,
      child: SafeArea(
          child: Padding(
        padding: EdgeInsets.symmetric(horizontal: 20),
        child: Forme(
          child: Builder(
            builder: (context) {
              return body;
            },
          ),
          key: formKey,
        ),
      )),
    );
  }
}
