import 'package:flutter/cupertino.dart';
import 'package:forme/forme.dart';

import 'package:forme_cupertino_fields/forme_cupertino_fields.dart';
import 'base_page.dart';

class CupertinoPickerFieldPage
    extends BasePage<int, FormeCupertinoPickerModel> {
  @override
  Widget get body {
    return Column(
      children: [
        FormeCupertinoPicker(
            listener: FormeValueFieldListener(
              onErrorChanged: (field, errorText) {
                controller.updateModel(FormeCupertinoPickerModel(
                  useMagnifier: errorText != null && errorText.invalid,
                  magnification: (errorText != null && errorText.invalid)
                      ? 1.0 + field.value / 100
                      : 1.0,
                ));
              },
              onValueChanged: (c, v) => print(v),
              autovalidateMode: AutovalidateMode.onUserInteraction,
              onValidate: FormeValidates.all([
                FormeValidates.notNull(),
                FormeValidates.min(100),
              ], errorText: 'value must bigger than 100'),
            ),
            name: name,
            initialValue: 50,
            itemExtent: 50,
            children:
                List<Widget>.generate(1000, (index) => Text(index.toString()))),
        Wrap(
          children: [
            createButton('change item extent to 30', () {
              controller.updateModel(FormeCupertinoPickerModel(itemExtent: 30));
            }),
            createButton('set children', () {
              controller.updateModel(
                FormeCupertinoPickerModel(
                    looping: true,
                    magnification: 1.2,
                    children: <Widget>[
                      CupertinoButton.filled(
                          child: Text('button'), onPressed: () {}),
                      CupertinoButton.filled(
                          child: Text('button'), onPressed: () {}),
                    ]),
              );
            }),
            createButton('use useMagnifier', () {
              controller.updateModel(FormeCupertinoPickerModel(
                useMagnifier: true,
                magnification: 1.3,
              ));
            }),
            createButton('update backgroundColor ', () {
              controller.updateModel(FormeCupertinoPickerModel(
                backgroundColor: CupertinoColors.systemPurple.withOpacity(0.3),
              ));
            }),
            createButton('update selection overlay ', () {
              controller.updateModel(FormeCupertinoPickerModel(
                selectionOverlay: Container(
                  color: CupertinoColors.systemPink.withOpacity(0.5),
                ),
              ));
            }),
            Builder(builder: (context) {
              bool lock = controller.model.locked ?? false;
              return createButton(lock ? 'enable scroll' : 'disable scroll',
                  () {
                controller.updateModel(FormeCupertinoPickerModel(
                  locked: !lock,
                ));
                (context as Element).markNeedsBuild();
              });
            }),
            createButton('update labelText', () async {
              updateLabel();
            }),
            createButton('set helper text', () {
              updateHelper();
            }),
            createButton('validate', () {
              controller.validate();
            }),
          ],
        )
      ],
    );
  }

  @override
  String get title => 'FormeCupertinoPicker';
}
