import 'package:flutter/cupertino.dart';

import 'package:forme/forme.dart';
import 'package:forme_cupertino_fields/forme_cupertino_fields.dart';

import 'base_page.dart';

class CupertinoTextFieldPage
    extends BasePage<String, FormeCupertinoTextFieldModel> {
  @override
  Widget get body {
    return Column(
      children: [
        FormeCupertinoTextField(
            listener: FormeValueFieldListener(
              autovalidateMode: AutovalidateMode.onUserInteraction,
              onValueChanged: (c, v) => print(v),
              onValidate: FormeValidates.size(
                  min: 10, errorText: 'length must bigger than 10'),
            ),
            name: name,
            decoratorBuilder:
                FormeCupertinoInputDecoratorBuilder(prefix: Text('email')),
            model: FormeCupertinoTextFieldModel(
              borderless: true,
              decoration:
                  FormeCupertinos.kDefaultCupertinoRoundedBorderDecoration,
              clearButtonMode: OverlayVisibilityMode.editing,
            )),
        Wrap(
          children: [
            createButton('update label', () {
              controller.decoratorController.update(
                  FormeCupertinoInputDecoratorModel(
                      prefix: Text('New Label Text')));
            }),
            createButton('update helper text', () {
              controller.decoratorController.update(
                  FormeCupertinoInputDecoratorModel(
                      helper: Text('helper text')));
            }),
            createButton('update decoration', () {
              controller.updateModel(FormeCupertinoTextFieldModel(
                decoration: BoxDecoration(
                    color: CupertinoDynamicColor.withBrightness(
                  color: CupertinoColors.systemRed,
                  darkColor: CupertinoColors.black,
                )),
              ));
            }),
            createButton('update prefix', () {
              controller.updateModel(FormeCupertinoTextFieldModel(
                prefix: Row(
                  mainAxisSize: MainAxisSize.min,
                  children: [Icon(CupertinoIcons.mail), Text('email')],
                ),
              ));
            }),
            createButton('update prefix mode', () {
              controller.updateModel(FormeCupertinoTextFieldModel(
                prefixMode: OverlayVisibilityMode.editing,
              ));
            }),
            createButton('update padding to 0', () {
              controller.updateModel(FormeCupertinoTextFieldModel(
                padding: EdgeInsets.all(0),
              ));
            }),
            createButton('update maxLength to 20', () {
              controller.updateModel(FormeCupertinoTextFieldModel(
                maxLength: 20,
              ));
            }),
            createButton('update lines to 5', () {
              controller.updateModel(FormeCupertinoTextFieldModel(
                maxLines: 5,
              ));
            }),
            createButton('update suffix', () {
              controller.updateModel(FormeCupertinoTextFieldModel(
                suffix: CupertinoButton(
                  onPressed: () {
                    controller.value = '';
                  },
                  child: Icon(CupertinoIcons.clear),
                ),
                suffixMode: OverlayVisibilityMode.editing,
              ));
            }),
          ],
        )
      ],
    );
  }

  @override
  String get title => 'FormeCupertinoTimerField';
}
