import 'package:flutter/cupertino.dart';

import 'package:forme/forme.dart';
import 'package:forme_cupertino_fields/src/field/cupertino_fields.dart';
import 'package:forme_cupertino_fields/src/field/forme_cupertino_slider.dart';
import 'package:forme_cupertino_fields/src/field/forme_cupertino_switch.dart';
import 'package:forme_cupertino_fields/src/forme_cupertino_decorator.dart';

import 'cupertino_segmented_control.dart';

class CupertinoDemoPage extends StatefulWidget {
  @override
  _DemoPageState createState() => _DemoPageState();
}

class _DemoPageState extends State<CupertinoDemoPage> {
  final FormeKey formKey = FormeKey();

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return CupertinoPageScaffold(
      child: SafeArea(
        child: SingleChildScrollView(
          child: Column(children: [
            Padding(
              padding: EdgeInsets.symmetric(horizontal: 20),
              child: createForm(),
            ),
            // for using snackbar here
            Builder(builder: (context) => createButtons(context)),
          ]),
        ),
      ),
    );
  }

  Widget createButtons(BuildContext context) {
    Wrap buttons = Wrap(
      children: [
        Builder(
          builder: (context) {
            return CupertinoButton(
                onPressed: () {
                  formKey.readOnly = !formKey.readOnly;
                  (context as Element).markNeedsBuild();
                },
                child: Text(formKey.readOnly
                    ? 'set form editable'
                    : 'set form readonly'));
          },
        ),
        CupertinoButton(
            onPressed: () {
              setState(() {});
            },
            child: Text('rebuild page')),
        CupertinoButton(
            onPressed: () async {
              FormeValidateSnapshot snapshot =
                  await formKey.validate(quietly: false);
              if (!snapshot.isValid) {
                FormeValueFieldController formeFieldController =
                    snapshot.firstInvalidField!.controller;
                formeFieldController.ensureVisible().then((value) {
                  formeFieldController.focusNode?.requestFocus();
                });
              } else {}
            },
            child: Text('validate')),
        CupertinoButton(
            onPressed: () {
              formKey.reset();
            },
            child: Text('reset')),
        CupertinoButton(
          onPressed: () {
            print(formKey.data);
          },
          child: Text('get form data'),
        ),
      ],
    );
    return buttons;
  }

  Map<String, Widget> createMap(String name) {
    return {
      'A': ReadOnlyWidget(
        text: 'A',
        formeKey: formKey,
        name: name,
      ),
      'B': ReadOnlyWidget(
        text: 'B',
        formeKey: formKey,
        name: name,
      ),
      'C': FormeValueFieldBuilder(
        builder: (context, controller) {
          return ValueListenableBuilder2<bool, FormeValidateError?>(
            controller.readOnlyListenable,
            controller.errorTextListenable,
            child: Text('C'),
            builder: (context, r, v, child) {
              if (r)
                return Text(
                  'C',
                  style: TextStyle(color: CupertinoColors.systemGrey),
                );
              if (v == null || !v.invalid) return child!;
              return ShakeWidget(
                child: child!,
                key: UniqueKey(),
              );
            },
          );
        },
      ),
    };
  }

  Widget createForm() {
    Widget child = Column(
      children: [
        CupertinoFormSection
            .insetGrouped(header: Text('Gesture Fields'), children: [
          FormeCupertinoSegmentedControl<String>(
            model: FormeCupertinoSegmentedControlModel<String>(
              padding: const EdgeInsets.symmetric(vertical: 10),
            ),
            decoratorBuilder: FormeCupertinoInputDecoratorBuilder(
              prefix: Text('control'),
            ),
            name: 'segmentedControl',
            chidren: createMap('segmentedControl'),
            listener: FormeValueFieldListener(
              autovalidateMode: AutovalidateMode.onUserInteraction,
              onValidate: FormeValidates.equals('C', errorText: 'pls select C'),
              onErrorChanged: (f, e) {
                if (e != null && e.validating) {
                  f.decoratorController
                      .update(FormeCupertinoInputDecoratorModel(
                          helper: Center(
                    child: CupertinoActivityIndicator(),
                  )));
                } else {
                  f.decoratorController.update(
                      FormeCupertinoInputDecoratorModel(
                          helper: SizedBox.shrink()));
                }
              },
              onAsyncValidate: (f, v) => Future.delayed(
                  Duration(milliseconds: 800), () => "pls select B"),
            ),
          ),
          FormeCupertinoSlidingSegmentedControl<String>(
            decoratorBuilder: FormeCupertinoInputDecoratorBuilder(
                prefix: Text('slidingControl')),
            name: 'segmentedSlidingControl',
            chidren: createMap('segmentedSlidingControl'),
            listener: FormeValueFieldListener(
              autovalidateMode: AutovalidateMode.onUserInteraction,
              onValidate: FormeValidates.equals('C', errorText: 'pls select C'),
            ),
          ),
          FormeCupertinoSlider(
            decoratorBuilder: FormeCupertinoInputDecoratorBuilder(
                prefix: Text('slider'),
                wrapper: (child) {
                  return Row(
                    children: [
                      Expanded(
                        child: child,
                      )
                    ],
                  );
                }),
            name: 'slider',
            min: 1,
            max: 100,
            listener: FormeValueFieldListener(
              autovalidateMode: AutovalidateMode.onUserInteraction,
              onValidate: FormeValidates.min(50, errorText: 'value must > 50'),
            ),
          ),
          FormeCupertinoPicker(
              listener: FormeValueFieldListener(
                onValidate: (f, v) {
                  if (v < 100) return 'value must bigger than 100';
                },
                onErrorChanged: (f, e) {
                  if (e != null) {
                    FormeValueFieldController<bool, FormeModel> switchField =
                        f.formeController.valueField('switch');
                    if (switchField.value) {
                      switchField.validate();
                    }
                  }
                },
                autovalidateMode: AutovalidateMode.onUserInteraction,
              ),
              decoratorBuilder: FormeCupertinoInputDecoratorBuilder(
                  prefix: Text('picker'),
                  wrapper: (child) => SizedBox(
                        height: 50,
                        width: double.infinity,
                        child: child,
                      )),
              name: 'cupertinoPicker',
              itemExtent: 30,
              children: List<Widget>.generate(1000, (index) {
                return FormeValueFieldBuilder(builder: (context, field) {
                  return ValueListenableBuilder<bool>(
                      valueListenable: field.readOnlyListenable,
                      builder: (context, v, child) {
                        if (v)
                          return Text(
                            index.toString(),
                            style: TextStyle(color: CupertinoColors.systemGrey),
                          );
                        return Text(index.toString());
                      });
                });
              })),
          FormeCupertinoSwitch(
            decoratorBuilder: FormeCupertinoInputDecoratorBuilder(
              prefix: Text('switch'),
            ),
            name: 'switch',
            listener: FormeValueFieldListener(
              onValidate:
                  FormeValidates.equals(true, errorText: 'accept and continue'),
              autovalidateMode: AutovalidateMode.onUserInteraction,
              onAsyncValidate: (f, v) =>
                  Future.delayed(Duration(seconds: 2), () {
                if (f.formeController.valueField('cupertinoPicker').value <
                    100) {
                  return 'pciker\'s value must bigger than 100';
                }
              }),
              onErrorChanged: (f, e) {
                if (e != null && e.validating) {
                  f.decoratorController
                      .update(FormeCupertinoInputDecoratorModel(
                          helper: Center(
                    child: CupertinoActivityIndicator(),
                  )));
                } else {
                  f.decoratorController.update(
                      FormeCupertinoInputDecoratorModel(
                          helper: SizedBox.shrink()));
                }
              },
            ),
          ),
        ]),
        CupertinoFormSection.insetGrouped(
          header: const Text('Input Fields'),
          children: [
            FormeCupertinoDateField(
              listener: FormeValueFieldListener(
                onValidate: FormeValidates.notNull(errorText: 'select a date!'),
                autovalidateMode: AutovalidateMode.onUserInteraction,
              ),
              name: 'dateTime',
              decoratorBuilder:
                  FormeCupertinoInputDecoratorBuilder(prefix: Text('datetime')),
              model: FormeCupertinoDateFieldModel(
                type: FormeDateTimeFieldType.DateTime,
                textFieldModel: FormeCupertinoTextFieldModel(
                  placeholder: 'tap to select',
                ),
              ),
            ),
            FormeCupertinoTimerField(
              listener: FormeValueFieldListener(
                onValidate:
                    FormeValidates.notNull(errorText: 'select a duration!'),
                autovalidateMode: AutovalidateMode.onUserInteraction,
              ),
              name: 'duration',
              decoratorBuilder:
                  FormeCupertinoInputDecoratorBuilder(prefix: Text('duration')),
              model: FormeCupertinoTimerFieldModel(
                textFieldModel: FormeCupertinoTextFieldModel(
                  placeholder: 'tap to select',
                ),
              ),
            ),
            FormeCupertinoTextField(
                listener: FormeValueFieldListener(
                  onValueChanged: (c, v) => print(v),
                  onValidate: FormeValidates.size(
                      min: 10, errorText: 'length must bigger than 10'),
                  autovalidateMode: AutovalidateMode.onUserInteraction,
                ),
                name: 'text',
                decoratorBuilder:
                    FormeCupertinoInputDecoratorBuilder(prefix: Text('text')),
                model: FormeCupertinoTextFieldModel(
                  clearButtonMode: OverlayVisibilityMode.editing,
                )),
            FormeCupertinoNumberField(
                name: 'number',
                decoratorBuilder:
                    FormeCupertinoInputDecoratorBuilder(prefix: Text('number')),
                listener: FormeValueFieldListener(
                  autovalidateMode: AutovalidateMode.onUserInteraction,
                  onValueChanged: (c, v) => print(v),
                  onValidate: FormeValidates.all([
                    FormeValidates.notNull(
                        errorText: 'number can not be empty'),
                    FormeValidates.min(10, errorText: 'must bigger than 10'),
                    FormeValidates.max(100, errorText: 'must smaller than 100')
                  ]),
                ),
                model: FormeCupertinoNumberFieldModel(
                    textFieldModel: FormeCupertinoTextFieldModel(
                  clearButtonMode: OverlayVisibilityMode.editing,
                ))),
          ],
        ),
        FormeColumn(
          name: 'column',
          children: [],
        ),
      ],
    );
    return Forme(
        child: child,
        key: formKey,
        onValueChanged: (a, b) {
          print(
              '${a.name}\'s value changed ... value: $b ... old value: ${a.oldValue}');
        },
        onErrorChanged: (a, b) {
          print('${a.name} ... error: ${b?.text}');
        },
        onFocusChanged: (a, b) {
          print('${a.name} ... focus: $b');
        });
  }
}
