import 'package:flutter/cupertino.dart';

import 'package:forme/forme.dart';

class FormeCupertinoSlider
    extends ValueField<double, FormeCupertinoSliderModel> {
  FormeCupertinoSlider({
    required String name,
    required double min,
    required double max,
    FormeCupertinoSliderModel? model,
    double? initialValue,
    bool readOnly = false,
    Key? key,
    FormeDecoratorBuilder<double>? decoratorBuilder,
    FormeValueFieldListener<double,
            FormeValueFieldController<double, FormeCupertinoSliderModel>>?
        listener,
    int? order,
    bool quietlyValidate = false,
  }) : super(
          quietlyValidate: quietlyValidate,
          listener: listener,
          decoratorBuilder: decoratorBuilder,
          key: key,
          readOnly: readOnly,
          name: name,
          initialValue: initialValue ?? min,
          model: (model ?? FormeCupertinoSliderModel())
              .copyWith(FormeCupertinoSliderModel(min: min, max: max)),
          builder: (baseState) {
            _FormeCupertinoSliderState state =
                baseState as _FormeCupertinoSliderState;
            return ValueListenableBuilder<double?>(
                valueListenable: state.notifier,
                builder: (context, _value, child) {
                  double min = state.model.min!;
                  double max = state.model.max!;
                  return Focus(
                    child: CupertinoSlider(
                        value: state.value,
                        min: min,
                        max: max,
                        onChangeStart: (v) {
                          state.requestFocus();
                          state.model.onChangeStart?.call(v);
                        },
                        onChangeEnd: (v) {
                          state.didChange(v);
                          state.model.onChangeEnd?.call(v);
                        },
                        activeColor: state.model.activeColor,
                        thumbColor:
                            state.model.thumbColor ?? CupertinoColors.white,
                        divisions: state.model.divisions ?? (max - min).floor(),
                        onChanged: state.readOnly
                            ? null
                            : (v) {
                                state.updateValue(v);
                                state.model.onChanged?.call(v);
                              }),
                    focusNode: state.focusNode,
                  );
                });
          },
        );

  @override
  _FormeCupertinoSliderState createState() => _FormeCupertinoSliderState();
}

class _FormeCupertinoSliderState
    extends ValueFieldState<double, FormeCupertinoSliderModel> {
  late final ValueNotifier<double?> notifier;

  updateValue(double value) {
    notifier.value = value;
  }

  @override
  void beforeInitiation() {
    super.beforeInitiation();
    notifier = FormeMountedValueNotifier(null, this);
  }

  @override
  double get initialValue {
    double defaultInitialValue = widget.initialValue;
    if (defaultInitialValue < model.min!) return model.min!;
    if (defaultInitialValue > model.max!) return model.max!;
    return defaultInitialValue;
  }

  @override
  double get value => notifier.value ?? super.value;

  @override
  void afterUpdateModel(
      FormeCupertinoSliderModel old, FormeCupertinoSliderModel current) {
    if (current.min != null && value < current.min!) setValue(current.min!);
    if (current.max != null && value > current.max!) setValue(current.max!);
  }

  @override
  FormeCupertinoSliderModel beforeSetModel(
      FormeCupertinoSliderModel old, FormeCupertinoSliderModel current) {
    if (current.min == null) {
      current = current.copyWith(FormeCupertinoSliderModel(min: old.min));
    }
    if (current.max == null) {
      current = current.copyWith(FormeCupertinoSliderModel(max: old.max));
    }
    return current;
  }

  @override
  void dispose() {
    notifier.dispose();
    super.dispose();
  }

  @override
  void onValueChanged(double? value) {
    notifier.value = null;
  }
}

class FormeCupertinoSliderModel extends FormeModel {
  final ValueChanged<double>? onChangeStart;
  final ValueChanged<double>? onChangeEnd;
  final ValueChanged<double>? onChanged;
  final double? min;
  final double? max;
  final int? divisions;
  final Color? activeColor;
  final Color? thumbColor;

  FormeCupertinoSliderModel({
    this.onChangeStart,
    this.onChangeEnd,
    this.min,
    this.max,
    this.divisions,
    this.activeColor,
    this.thumbColor,
    this.onChanged,
  });

  @override
  FormeCupertinoSliderModel copyWith(FormeModel oldModel) {
    FormeCupertinoSliderModel old = oldModel as FormeCupertinoSliderModel;

    return FormeCupertinoSliderModel(
      onChangeStart: onChangeStart ?? old.onChangeStart,
      onChangeEnd: onChangeEnd ?? old.onChangeEnd,
      min: min ?? old.min,
      max: max ?? old.max,
      divisions: divisions ?? old.divisions,
      activeColor: activeColor ?? old.activeColor,
      thumbColor: thumbColor ?? old.thumbColor,
      onChanged: onChanged ?? old.onChanged,
    );
  }
}
