import 'package:flutter/cupertino.dart';
import 'package:flutter/gestures.dart';
import 'package:forme/forme.dart';

class FormeCupertinoSwitch extends ValueField<bool, FormeCupertinoSwitchModel> {
  FormeCupertinoSwitch({
    bool initialValue = false,
    required String name,
    bool readOnly = false,
    Widget? label,
    FormeCupertinoSwitchModel? model,
    Key? key,
    FormeDecoratorBuilder<bool>? decoratorBuilder,
    FormeValueFieldListener<bool,
            FormeValueFieldController<bool, FormeCupertinoSwitchModel>>?
        listener,
    int? order,
    bool quietlyValidate = false,
  }) : super(
          quietlyValidate: quietlyValidate,
          listener: listener,
          decoratorBuilder: decoratorBuilder,
          key: key,
          model: model ?? FormeCupertinoSwitchModel(),
          readOnly: readOnly,
          name: name,
          initialValue: initialValue,
          builder: (state) {
            bool readOnly = state.readOnly;
            bool value = state.value;
            return CupertinoSwitch(
              value: value,
              onChanged: readOnly
                  ? null
                  : (v) {
                      state.didChange(v);
                      state.requestFocus();
                    },
              activeColor: state.model.activeColor,
              trackColor: state.model.trackColor,
              dragStartBehavior:
                  state.model.dragStartBehavior ?? DragStartBehavior.start,
            );
          },
        );
}

class FormeCupertinoSwitchModel extends FormeModel {
  final Color? activeColor;
  final Color? trackColor;
  final DragStartBehavior? dragStartBehavior;

  FormeCupertinoSwitchModel({
    this.activeColor,
    this.trackColor,
    this.dragStartBehavior,
  });

  FormeCupertinoSwitchModel copyWith(FormeModel oldModel) {
    FormeCupertinoSwitchModel old = oldModel as FormeCupertinoSwitchModel;
    return FormeCupertinoSwitchModel(
      activeColor: activeColor ?? old.activeColor,
      trackColor: trackColor ?? old.trackColor,
      dragStartBehavior: dragStartBehavior ?? old.dragStartBehavior,
    );
  }
}
