import 'dart:ui';

import 'package:flutter/cupertino.dart';
import 'package:flutter/gestures.dart';

import 'package:flutter/services.dart';
import 'package:forme/forme.dart';

import 'package:forme_cupertino_fields/forme_cupertino_fields.dart';

class FormeCupertinoTextField extends BaseValueField<String,
    FormeCupertinoTextFieldModel, FormeCupertinoTextFieldController> {
  FormeCupertinoTextField({
    String? initialValue,
    required String name,
    bool readOnly = false,
    FormeCupertinoTextFieldModel? model,
    Key? key,
    FormeDecoratorBuilder<String>? decoratorBuilder,
    int? maxLines = 1,
    FormeValueFieldListener<String, FormeCupertinoTextFieldController>?
        listener,
    int? order,
    bool quietlyValidate = false,
  }) : super(
            quietlyValidate: quietlyValidate,
            order: order,
            listener: listener,
            decoratorBuilder: decoratorBuilder,
            key: key,
            model: (model ?? FormeCupertinoTextFieldModel())
                .copyWith(FormeCupertinoTextFieldModel(maxLines: maxLines)),
            name: name,
            readOnly: readOnly,
            initialValue: initialValue ?? '',
            builder: (baseState) {
              _FormeCupertinoTextFieldState state =
                  baseState as _FormeCupertinoTextFieldState;
              bool readOnly = state.readOnly;
              FormeCupertinoTextFieldModel model = FormeCupertinoTextFieldModel(
                      onChanged: (v) {
                        state.didChange(v);
                        state.model.onChanged?.call(v);
                      },
                      onTap: readOnly ? () {} : state.model.onTap,
                      readOnly: readOnly)
                  .copyWith(state.model);
              return FormeCupertinos.cupertinoTextField(
                  focusNode: state.focusNode,
                  textEditingController: state.textEditingController,
                  model: model);
            });

  @override
  _FormeCupertinoTextFieldState createState() =>
      _FormeCupertinoTextFieldState();
}

class _FormeCupertinoTextFieldState extends BaseValueFieldState<String,
    FormeCupertinoTextFieldModel, FormeCupertinoTextFieldController> {
  late final TextEditingController textEditingController;

  @override
  FormeCupertinoTextField get widget => super.widget as FormeCupertinoTextField;

  @override
  void beforeInitiation() {
    super.beforeInitiation();
    textEditingController = TextEditingController(text: initialValue);
  }

  @override
  void onValueChanged(String? value) {
    if (textEditingController.text != value)
      textEditingController.text = value!;
  }

  @override
  void dispose() {
    textEditingController.dispose();
    super.dispose();
  }

  @override
  FormeCupertinoTextFieldController createFormeFieldController() {
    return FormeCupertinoTextFieldController(
        super.defaultFormeValueFieldController(), this);
  }
}

class FormeCupertinoTextFieldController
    extends FormeValueFieldControllerDelegate<String,
        FormeCupertinoTextFieldModel> {
  final FormeValueFieldController<String, FormeCupertinoTextFieldModel>
      delegate;
  final _FormeCupertinoTextFieldState _state;

  /// this textEditingController is not the underlying `TextEditingController` that `FormeTextField` used
  ///
  /// **this controller should regarded as write only , though you still can change `FormeTextField`'s value via [TextEditingController.value]**
  final TextEditingController textEditingController;
  FormeCupertinoTextFieldController(this.delegate, this._state)
      : this.textEditingController = TextEditingController.fromValue(
            _state.textEditingController.value) {
    this.textEditingController.addListener(() {
      _state.didChange(this.textEditingController.text);
      _state.textEditingController.value = this.textEditingController.value;
    });
  }

  void selectAll() {
    textEditingController.selection =
        FormeUtils.selection(0, textEditingController.text.length);
  }

  /// get textEditingValue **from underlying controller that `FormeTextField` used**
  TextEditingValue get textEditingValue => _state.textEditingController.value;

  /// get selection **from underlying controller that `FormeTextField` used**
  TextSelection get selection => _state.textEditingController.selection;
}

class FormeCupertinoTextFieldModel extends FormeModel {
  final BoxDecoration? decoration;
  final EdgeInsetsGeometry? padding;
  final String? placeholder;
  final TextStyle? placeholderStyle;
  final Widget? prefix;
  final OverlayVisibilityMode? prefixMode;
  final Widget? suffix;
  final OverlayVisibilityMode? suffixMode;
  final OverlayVisibilityMode? clearButtonMode;
  final TextInputType? keyboardType;
  final TextInputAction? textInputAction;
  final TextCapitalization? textCapitalization;
  final TextStyle? style;
  final StrutStyle? strutStyle;
  final TextAlign? textAlign;
  final ToolbarOptions? toolbarOptions;
  final TextAlignVertical? textAlignVertical;
  final bool? showCursor;
  final bool? autofocus;
  final String? obscuringCharacter;
  final bool? obscureText;
  final bool? autocorrect;
  final SmartDashesType? smartDashesType;
  final SmartQuotesType? smartQuotesType;
  final bool? enableSuggestions;
  final int? maxLines;
  final int? minLines;
  final bool? expands;
  final int? maxLength;
  final MaxLengthEnforcement? maxLengthEnforcement;
  final ValueChanged<String>? onChanged;
  final VoidCallback? onEditingComplete;
  final ValueChanged<String>? onSubmitted;
  final List<TextInputFormatter>? inputFormatters;
  final bool? enabled;
  final double? cursorWidth;
  final double? cursorHeight;
  final Radius? cursorRadius;
  final Color? cursorColor;
  final BoxHeightStyle? selectionHeightStyle;
  final BoxWidthStyle? selectionWidthStyle;
  final Brightness? keyboardAppearance;
  final EdgeInsets? scrollPadding;
  final bool? enableInteractiveSelection;
  final TextSelectionControls? selectionControls;
  final DragStartBehavior? dragStartBehavior;
  final ScrollController? scrollController;
  final ScrollPhysics? scrollPhysics;
  final GestureTapCallback? onTap;
  final Iterable<String>? autofillHints;
  final bool? readOnly;
  final bool? borderless;

  const FormeCupertinoTextFieldModel({
    this.padding,
    this.placeholder,
    this.placeholderStyle,
    this.prefix,
    this.prefixMode,
    this.suffix,
    this.suffixMode,
    this.clearButtonMode,
    this.selectionControls,
    this.scrollController,
    this.keyboardType,
    this.textInputAction,
    this.textCapitalization,
    this.style,
    this.strutStyle,
    this.textAlign,
    this.textAlignVertical,
    this.showCursor,
    this.obscuringCharacter,
    this.obscureText,
    this.autocorrect,
    this.smartDashesType,
    this.smartQuotesType,
    this.enableSuggestions,
    this.maxLines,
    this.minLines,
    this.expands,
    this.maxLength,
    this.maxLengthEnforcement,
    this.cursorWidth,
    this.cursorHeight,
    this.cursorRadius,
    this.cursorColor,
    this.selectionHeightStyle,
    this.selectionWidthStyle,
    this.keyboardAppearance,
    this.scrollPadding,
    this.dragStartBehavior,
    this.scrollPhysics,
    this.autofillHints,
    this.decoration,
    this.autofocus,
    this.toolbarOptions,
    this.enableInteractiveSelection,
    this.enabled,
    this.onEditingComplete,
    this.inputFormatters,
    this.onTap,
    this.onChanged,
    this.onSubmitted,
    this.readOnly,
    this.borderless,
  });

  static FormeCupertinoTextFieldModel? copy(FormeCupertinoTextFieldModel? old,
      FormeCupertinoTextFieldModel? current) {
    if (current == null) return old;
    if (old == null) return current;
    return current.copyWith(old);
  }

  @override
  FormeCupertinoTextFieldModel copyWith(FormeModel oldModel) {
    FormeCupertinoTextFieldModel old = oldModel as FormeCupertinoTextFieldModel;
    return FormeCupertinoTextFieldModel(
      padding: padding ?? old.padding,
      placeholder: placeholder ?? old.placeholder,
      placeholderStyle: placeholderStyle ?? old.placeholderStyle,
      prefix: prefix ?? old.prefix,
      prefixMode: prefixMode ?? old.prefixMode,
      suffix: suffix ?? old.suffix,
      suffixMode: suffixMode ?? old.suffixMode,
      clearButtonMode: clearButtonMode ?? old.clearButtonMode,
      keyboardType: keyboardType ?? old.keyboardType,
      autofocus: autofocus ?? old.autofocus,
      maxLines: maxLines ?? old.maxLines,
      minLines: minLines ?? old.minLines,
      maxLength: maxLength ?? old.maxLength,
      style: style ?? old.style,
      toolbarOptions: toolbarOptions ?? old.toolbarOptions,
      textInputAction: textInputAction ?? old.textInputAction,
      textCapitalization: textCapitalization ?? old.textCapitalization,
      obscureText: obscureText ?? old.obscureText,
      strutStyle: strutStyle ?? old.strutStyle,
      textAlign: textAlign ?? old.textAlign,
      textAlignVertical: textAlignVertical ?? old.textAlignVertical,
      showCursor: showCursor ?? old.showCursor,
      obscuringCharacter: obscuringCharacter ?? old.obscuringCharacter,
      autocorrect: autocorrect ?? old.autocorrect,
      smartDashesType: smartDashesType ?? old.smartDashesType,
      smartQuotesType: smartQuotesType ?? old.smartQuotesType,
      enableSuggestions: enableSuggestions ?? old.enableSuggestions,
      expands: expands ?? old.expands,
      maxLengthEnforcement: maxLengthEnforcement ?? old.maxLengthEnforcement,
      cursorWidth: cursorWidth ?? old.cursorWidth,
      cursorHeight: cursorHeight ?? old.cursorHeight,
      cursorRadius: cursorRadius ?? old.cursorRadius,
      cursorColor: cursorColor ?? old.cursorColor,
      selectionHeightStyle: selectionHeightStyle ?? old.selectionHeightStyle,
      selectionWidthStyle: selectionWidthStyle ?? old.selectionWidthStyle,
      keyboardAppearance: keyboardAppearance ?? old.keyboardAppearance,
      scrollPadding: scrollPadding ?? old.scrollPadding,
      dragStartBehavior: dragStartBehavior ?? old.dragStartBehavior,
      scrollPhysics: scrollPhysics ?? old.scrollPhysics,
      autofillHints: autofillHints ?? old.autofillHints,
      enableInteractiveSelection:
          enableInteractiveSelection ?? old.enableInteractiveSelection,
      enabled: enabled ?? old.enabled,
      onEditingComplete: onEditingComplete ?? old.onEditingComplete,
      inputFormatters: inputFormatters ?? old.inputFormatters,
      onTap: onTap ?? old.onTap,
      onChanged: onChanged ?? old.onChanged,
      onSubmitted: onSubmitted ?? old.onSubmitted,
      readOnly: readOnly ?? old.readOnly,
      scrollController: scrollController ?? old.scrollController,
      selectionControls: selectionControls ?? old.selectionControls,
      decoration: decoration ?? old.decoration,
      borderless: borderless ?? old.borderless,
    );
  }
}
