import 'package:flutter/cupertino.dart';
import 'package:forme/forme.dart';

/// this builder will decorate current field with [CupertinoFormRow]
class FormeCupertinoInputDecoratorBuilder<T>
    implements FormeDecoratorBuilder<T> {
  final Widget? prefix;
  final EdgeInsets? padding;
  final Widget? helper;
  final Widget Function(Widget child)? wrapper;
  const FormeCupertinoInputDecoratorBuilder({
    this.wrapper,
    this.prefix,
    this.padding,
    this.helper,
  });
  @override
  Widget build(
    FormeValueFieldController<T, FormeModel> controller,
    Widget child,
  ) {
    return FormeCupertinoInputDecorator(
      model: FormeCupertinoInputDecoratorModel(
          helper: helper, padding: padding, prefix: prefix),
      controller: controller,
      child: child,
      wrapper: wrapper,
    );
  }
}

class FormeCupertinoInputDecorator<T> extends StatelessWidget {
  final FormeCupertinoInputDecoratorModel model;
  final Widget child;
  final Widget Function(Widget child)? wrapper;
  final FormeValueFieldController<T, FormeModel> controller;

  const FormeCupertinoInputDecorator({
    required this.model,
    this.wrapper,
    required this.controller,
    required this.child,
  });

  @override
  Widget build(BuildContext context) {
    Widget child = wrapper == null ? this.child : wrapper!(this.child);
    FormeCupertinoInputDecoratorModel model =
        (controller.decoratorController.currentModel ??
                FormeCupertinoInputDecoratorModel())
            .copyWith(this.model) as FormeCupertinoInputDecoratorModel;
    if (FormeKey.of(context).quietlyValidate) {
      return CupertinoFormRow(
        child: child,
        helper: model.helper,
        padding: model.padding,
        prefix: model.prefix,
      );
    }
    return ValueListenableBuilder<FormeValidateError?>(
      valueListenable: controller.errorTextListenable,
      builder: (context, FormeValidateError? error, _child) {
        return CupertinoFormRow(
          child: child,
          helper: model.helper,
          padding: model.padding,
          prefix: model.prefix,
          error: error == null
              ? null
              : error.invalid
                  ? Text(error.text!)
                  : null,
        );
      },
    );
  }
}

class FormeCupertinoInputDecoratorModel extends FormeModel {
  final Widget? prefix;
  final EdgeInsets? padding;
  final Widget? helper;

  const FormeCupertinoInputDecoratorModel({
    this.prefix,
    this.padding,
    this.helper,
  });

  @override
  FormeCupertinoInputDecoratorModel copyWith(FormeModel oldModel) {
    FormeCupertinoInputDecoratorModel old =
        oldModel as FormeCupertinoInputDecoratorModel;
    return FormeCupertinoInputDecoratorModel(
      prefix: prefix ?? old.prefix,
      padding: padding ?? old.padding,
      helper: helper ?? old.helper,
    );
  }
}
