import 'dart:ui';

import 'package:flutter/cupertino.dart';
import 'package:flutter/gestures.dart';

import 'package:forme_cupertino_fields/forme_cupertino_fields.dart';

// Value inspected from Xcode 11 & iOS 13.0 Simulator.
const BorderSide _kDefaultRoundedBorderSide = BorderSide(
  color: CupertinoDynamicColor.withBrightness(
    color: Color(0x33000000),
    darkColor: Color(0x33FFFFFF),
  ),
  style: BorderStyle.solid,
  width: 0.0,
);
const Border _kDefaultRoundedBorder = Border(
  top: _kDefaultRoundedBorderSide,
  bottom: _kDefaultRoundedBorderSide,
  left: _kDefaultRoundedBorderSide,
  right: _kDefaultRoundedBorderSide,
);

class FormeCupertinos {
  FormeCupertinos._();

  static const BoxDecoration kDefaultCupertinoRoundedBorderDecoration =
      BoxDecoration(
    color: CupertinoDynamicColor.withBrightness(
      color: CupertinoColors.white,
      darkColor: CupertinoColors.black,
    ),
    border: _kDefaultRoundedBorder,
    borderRadius: BorderRadius.all(Radius.circular(5.0)),
  );

  /// show a bottom sheet model
  static Future<T?> showFormeCupertinoModalPopup<T>({
    required BuildContext context,
    required FormeCupertinoModelPopupData? data,
    required WidgetBuilder builder,
  }) {
    data?.beforeOpen?.call();
    return showCupertinoModalPopup<T>(
      context: context,
      builder: builder,
      barrierColor: data?.barrierColor ?? kCupertinoModalBarrierColor,
      filter: data?.filter,
      barrierDismissible: data?.barrierDismissible ?? true,
      useRootNavigator: data?.useRootNavigator ?? true,
      semanticsDismissible: data?.semanticsDismissible,
      routeSettings: data?.routeSettings,
    ).whenComplete(() {
      data?.afterClose?.call();
    });
  }

  static cupertinoTextField({
    required FocusNode focusNode,
    required TextEditingController textEditingController,
    FormeCupertinoTextFieldModel model = const FormeCupertinoTextFieldModel(),
  }) {
    bool borderless = model.borderless ?? true;
    if (borderless)
      return _borderlessCupertinoTextField(
        focusNode: focusNode,
        textEditingController: textEditingController,
        model: model,
      );
    return CupertinoTextField(
      focusNode: focusNode,
      controller: textEditingController,
      decoration: model.decoration,
      padding: model.padding ?? const EdgeInsets.all(6.0),
      placeholder: model.placeholder,
      placeholderStyle: model.placeholderStyle ??
          const TextStyle(
            fontWeight: FontWeight.w400,
            color: CupertinoColors.placeholderText,
          ),
      prefix: model.prefix,
      prefixMode: model.prefixMode ?? OverlayVisibilityMode.always,
      suffix: model.suffix,
      suffixMode: model.suffixMode ?? OverlayVisibilityMode.always,
      clearButtonMode: model.clearButtonMode ?? OverlayVisibilityMode.never,
      keyboardType: model.keyboardType,
      textInputAction: model.textInputAction,
      textCapitalization: model.textCapitalization ?? TextCapitalization.none,
      style: model.style,
      strutStyle: model.strutStyle,
      textAlign: model.textAlign ?? TextAlign.start,
      textAlignVertical: model.textAlignVertical,
      readOnly: model.readOnly ?? false,
      toolbarOptions: model.toolbarOptions,
      showCursor: model.showCursor,
      autofocus: model.autofocus ?? false,
      obscuringCharacter: model.obscuringCharacter ?? '•',
      obscureText: model.obscureText ?? false,
      autocorrect: model.autocorrect ?? true,
      smartDashesType: model.smartDashesType,
      smartQuotesType: model.smartQuotesType,
      enableSuggestions: model.enableSuggestions ?? true,
      maxLines: model.maxLines,
      minLines: model.minLines,
      expands: model.expands ?? false,
      maxLength: model.maxLength,
      maxLengthEnforcement: model.maxLengthEnforcement,
      onChanged: model.onChanged,
      onEditingComplete: model.onEditingComplete,
      onSubmitted: model.onSubmitted,
      inputFormatters: model.inputFormatters,
      cursorWidth: model.cursorWidth ?? 2.0,
      cursorHeight: model.cursorHeight,
      cursorRadius: model.cursorRadius ?? const Radius.circular(2.0),
      cursorColor: model.cursorColor,
      selectionHeightStyle: model.selectionHeightStyle ?? BoxHeightStyle.tight,
      selectionWidthStyle: model.selectionWidthStyle ?? BoxWidthStyle.tight,
      keyboardAppearance: model.keyboardAppearance,
      scrollPadding: model.scrollPadding ?? const EdgeInsets.all(20.0),
      dragStartBehavior: model.dragStartBehavior ?? DragStartBehavior.start,
      enableInteractiveSelection: model.enableInteractiveSelection ?? true,
      selectionControls: model.selectionControls,
      onTap: model.onTap,
      scrollController: model.scrollController,
      scrollPhysics: model.scrollPhysics,
      autofillHints: model.autofillHints,
    );
  }

  static _borderlessCupertinoTextField({
    required FocusNode focusNode,
    required TextEditingController textEditingController,
    FormeCupertinoTextFieldModel model = const FormeCupertinoTextFieldModel(),
  }) {
    return CupertinoTextField.borderless(
      focusNode: focusNode,
      controller: textEditingController,
      decoration: model.decoration,
      padding: model.padding ?? const EdgeInsets.all(6.0),
      placeholder: model.placeholder,
      placeholderStyle: model.placeholderStyle ??
          const TextStyle(
            fontWeight: FontWeight.w400,
            color: CupertinoColors.placeholderText,
          ),
      prefix: model.prefix,
      prefixMode: model.prefixMode ?? OverlayVisibilityMode.always,
      suffix: model.suffix,
      suffixMode: model.suffixMode ?? OverlayVisibilityMode.always,
      clearButtonMode: model.clearButtonMode ?? OverlayVisibilityMode.never,
      keyboardType: model.keyboardType,
      textInputAction: model.textInputAction,
      textCapitalization: model.textCapitalization ?? TextCapitalization.none,
      style: model.style,
      strutStyle: model.strutStyle,
      textAlign: model.textAlign ?? TextAlign.start,
      textAlignVertical: model.textAlignVertical,
      readOnly: model.readOnly ?? false,
      toolbarOptions: model.toolbarOptions,
      showCursor: model.showCursor,
      autofocus: model.autofocus ?? false,
      obscuringCharacter: model.obscuringCharacter ?? '•',
      obscureText: model.obscureText ?? false,
      autocorrect: model.autocorrect ?? true,
      smartDashesType: model.smartDashesType,
      smartQuotesType: model.smartQuotesType,
      enableSuggestions: model.enableSuggestions ?? true,
      maxLines: model.maxLines,
      minLines: model.minLines,
      expands: model.expands ?? false,
      maxLength: model.maxLength,
      maxLengthEnforcement: model.maxLengthEnforcement,
      onChanged: model.onChanged,
      onEditingComplete: model.onEditingComplete,
      onSubmitted: model.onSubmitted,
      inputFormatters: model.inputFormatters,
      cursorWidth: model.cursorWidth ?? 2.0,
      cursorHeight: model.cursorHeight,
      cursorRadius: model.cursorRadius ?? const Radius.circular(2.0),
      cursorColor: model.cursorColor,
      selectionHeightStyle: model.selectionHeightStyle ?? BoxHeightStyle.tight,
      selectionWidthStyle: model.selectionWidthStyle ?? BoxWidthStyle.tight,
      keyboardAppearance: model.keyboardAppearance,
      scrollPadding: model.scrollPadding ?? const EdgeInsets.all(20.0),
      dragStartBehavior: model.dragStartBehavior ?? DragStartBehavior.start,
      enableInteractiveSelection: model.enableInteractiveSelection ?? true,
      selectionControls: model.selectionControls,
      onTap: model.onTap,
      scrollController: model.scrollController,
      scrollPhysics: model.scrollPhysics,
      autofillHints: model.autofillHints,
    );
  }
}

class FormeCupertinoModelPopupData {
  final ImageFilter? filter;
  final Color? barrierColor;
  final bool? barrierDismissible;
  final bool? useRootNavigator;
  final bool? semanticsDismissible;
  final RouteSettings? routeSettings;

  final VoidCallback? beforeOpen;
  final VoidCallback? afterClose;

  const FormeCupertinoModelPopupData({
    this.filter,
    this.barrierColor,
    this.barrierDismissible,
    this.useRootNavigator,
    this.semanticsDismissible,
    this.routeSettings,
    this.beforeOpen,
    this.afterClose,
  });

  static FormeCupertinoModelPopupData? copy(FormeCupertinoModelPopupData? old,
      FormeCupertinoModelPopupData? current) {
    if (current == null) return old;
    if (old == null) return current;
    return FormeCupertinoModelPopupData(
      barrierColor: current.barrierColor ?? old.barrierColor,
      filter: current.filter ?? old.filter,
      barrierDismissible: current.barrierDismissible ?? old.barrierDismissible,
      useRootNavigator: current.useRootNavigator ?? old.useRootNavigator,
      semanticsDismissible:
          current.semanticsDismissible ?? old.semanticsDismissible,
      routeSettings: current.routeSettings ?? old.routeSettings,
      beforeOpen: current.beforeOpen ?? old.beforeOpen,
      afterClose: current.afterClose ?? old.afterClose,
    );
  }
}
